#ifndef DUPDUMP_STORE_H
#define DUPDUMP_STORE_H 1

#include <openssl/sha.h>

/*
 * This is used to not visit the same file/dir two times
 */
struct dev_ino
{
	dev_t			dev;
	ino_t			ino;
	struct dev_ino		*next;
};

struct file_node
{
	char			*name;
	unsigned long long	size;
	unsigned char		sha1_first[SHA_DIGEST_LENGTH];
	unsigned char		sha1_full[SHA_DIGEST_LENGTH];
	unsigned char		skip_compare:1;
	unsigned char		no_dup_possible:1;
	unsigned char		do_not_dump:1;
	unsigned char		unique:1;
	unsigned char		left:1;
	int			level;
	dev_t			dev;
	ino_t			ino;
	struct file_node	*next;
	struct file_node	*hash_next;
	struct dir_node		*parent;
	struct file_node	*duplicates;
	time_t			mtime;
};

struct dir_node
{
	char			*name;
	unsigned char		sha1[SHA_DIGEST_LENGTH];
	unsigned char		file_names_sha1[SHA_DIGEST_LENGTH];
	unsigned char		no_dup_possible:1;
	unsigned char		do_not_dump:1;
	unsigned char		left:1;

	int			level;
	dev_t			dev;
	ino_t			ino;
	struct dir_node		*subdirs;
	struct dir_node		*next_sibling; /* Link subdirs on the same level */
	struct file_node	*files;
	unsigned int		no_of_files;
	struct dir_node		*parent;
	struct dir_node		*hash_next;	/* in the last phase, here we store duplicates */
	time_t			mtime;
};


extern void			set_debug(const unsigned int level);
extern void			set_out(FILE *out);
extern void			dump_stats(void);
extern int			file_add(const char *file, const struct stat *s,
					const int level);
extern int			dir_add(const char *dir, const struct stat *s,
					const int level);
extern void			dump_files(void);
extern int			file_find_dups(void);
extern int			dir_find_dups(void);
extern struct dir_node		*dir_get_current(const unsigned int level);

extern int			dev_ino_seen(const int type,
					const dev_t dev, const ino_t ino);
extern void			dev_ino_seen_clean(void);

extern void			dump_dirs(void);

extern void			dump_duplicates(const unsigned long long min_size,
					const unsigned int zero);

#endif
